#!/usr/bin/env python
# -*- mode:Python; tab-width: 4 -*-
"""Extract information about links from a file.

Processes the given input files and extracts their links. Then dumps
them on the screen, grouping them in local and external links.
Works with HTML code, and if nothing is found, falls back to CSS
link extraction.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2 of the License,
or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""

__author__ = "Grzegorz Adam Hankiewicz <gradha@efaber.net>"
__date__ = "May 2003"
__version__ = "$Revision: 1.13 $"
__credits__ = ""

import getopt
import sys
import urlparse
import os.path
import html_parser
import css_parser


def process_data(data):
    """Processes HTML/CSS data and returns links found in it.

    data is considered to be a big string of HTML/CSS code. The
    function returns a tuple with two lists, the first contains
    the local relative links and the second one contains external
    links.  The returned links are guaranteed to be unique, but
    not necessarily sorted.
    """
    local = {}
    external = {}
    extractor = html_parser.Link_extractor(data, 1)
    if not extractor.get_links():
        extractor = css_parser.Link_extractor(data, 1)
    for link in extractor.get_links():
        try:
            t1, t2, t3, t4, t5, t6 = urlparse.urlparse(link)
            if t3[0] == '/':
                external[link] = 1
            elif not t1 or t1 == "file":
                local[link] = 1
            else:
                external[link] = 1
        except IndexError:
            external[link] = 1
    # get non repeated links, sort and print them
    return (local.keys(), external.keys())


def process_file(filename):
    """Processes an HTML/CSS file and returns links found in it.

    Opens filename, which is presumed to be HTML/CSS and tries to
    extract from it all the links. The return value of this function
    is the same as the return value of process_data. If the file
    could not be read, two empty listes will be returned.
    """
    file = open(filename, "rb")
    try:
        try:
            data = file.read()
        except IOError:
            print "Couldn't read %s" % filename
            return ([], [])
    finally:
        file.close()

    return process_data(data)


def print_link_result(local, external, filename = ""):
    """Pretty prints two lists of links.

    Given two lists, with local and external links, goes through
    them printing them with the format "filename:(LOC|EXT):link". If
    a local link doesn't exist, this will be notified.
    """
    local.sort()
    external.sort()
    preffix = "%s:LOC:" % filename
    for f in local:
        print preffix, f,
        file = urlparse.urljoin(filename, html_parser.split_url(f)[0])
        if os.path.isfile(file):
            print ""
        else:
            print "(doesn't exist)"
            
    preffix = "%s:EXT:" % filename
    for f in external:
        print preffix, f


def print_usage_information(exit_code, binary_name = "link_grep.py"):
    """Prints usage information and terminates execution."""
    print "This program extracts all internal and external references from an"
    print "HTML/CSS file and prints them sorted in two categories."
    print "Additionally, local files are checked if they really exist.\n"
    print "Usage %s [-hv] file_1 file_2 ... file_n" % binary_name
    print "\tYou can also pipe HTML/CSS code directly.\n"
    print " -h, --help      Show this screen"
    print " -v, --version   Print revision number"
    sys.exit(exit_code)


def main(binary_name = "link_grep.py"):
    """Entry point of the appliaction. Processes command line arguments."""
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hv", ["help", "version"])
    except getopt.GetoptError:
        # print help information and exit:
        print_usage_information(2)
        
    for param, value in opts:
        if param in ("-h", "--help"):
            print_usage_information(0)
        if param in ("-v", "--version"):
            print binary_name, __version__[11:-1]
            sys.exit(0)

    try:
        if not args:
            local, external = process_data(sys.stdin.read())
            print_link_result(local, external, "stdin")
        else:
            for filename in args:
                local, external = process_file(filename)
                print_link_result(local, external, filename)
    except KeyboardInterrupt:
        pass
    
if __name__ == "__main__":
    main(os.path.basename(sys.argv[0]))
